/*
 * Copyright (c) 2008 Jiri Svoboda
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup debug
 * @{
 */
/** @file
 */

#include <stdlib.h>
#include <adt/list.h>
#include <assert.h>
#include <futex.h>
#include <async.h>

#include "include/arch.h"
#include "main.h"
#include "dthread.h"

static int last_thread_id = 0;

link_t dthreads;		/* List of application's threads */
dthread_t *cwt;			/* Current working thread */

dthread_t *dthread_new(thash_t hash)
{
	dthread_t *dt;

	dt = malloc(sizeof(dthread_t));
	if (!dt) {
		exit(1);
	}

	dt->id = ++last_thread_id;
	dt->hash = hash;
	dt->fid = 0;
	dt->stopped = 0;

	arch_dthread_initialize(dt);

	if (!cwt) cwt = dt;

	list_append(&dt->link, &dthreads);

	return dt;
}

void dthread_delete(dthread_t *dt)
{
	list_remove(&dt->link);
	free(dt);
}

/*
 * Get dthread struct serviced by fibril fid.
 */
dthread_t *dthread_get_by_fid(fid_t fid)
{
	link_t *cur;
	dthread_t *dt;

	dt = NULL;
	for (cur = dthreads.next; cur != &dthreads; cur = cur->next) {
		dt = list_get_instance(cur, dthread_t, link);
		if (dt->fid == fid) return dt;
	}	

	return NULL;
}

/*
 * Get dthread struct for dthread serviced by the current fibril.
 */
dthread_t *dthread_get(void)
{
	return dthread_get_by_fid(fibril_get_id());
}

void dthread_stop_me(void)
{
	dthread_t *dt;

	dt = dthread_get();
	assert(dt);

	futex_down(&async_futex);

	dt->stopped = 1;
	fibril_switch(FIBRIL_TO_MANAGER);

	futex_up(&async_futex);
}

void dthread_resume(dthread_t *dt)
{
	if (!dt->stopped) return;

	futex_down(&async_futex);

	fibril_add_ready(dt->fid);
	dt->stopped = 0;

	futex_up(&async_futex);
}

unsigned dthread_get_pc(dthread_t *dt)
{
	static istate_t istate;
	int rc;

	rc = udebug_regs_read(app_phone, dt->hash, &istate);
	if (rc < 0) {
		printf("failed reading registers (%d)\n", rc);
		return 0;
	}

	return istate_get_pc(&istate);
}

/** @}
 */
